/*
 *  MACTOPHY.c
 *
 *  Description :   This module is the interface netween MAC and PHY.
 *                  All the initialization, setting operation, detecting
 *                  active media, dynamic sensing routines are supported
 *                  in this module.
 *
 *  Created by Ken Vong
 *
 *  Copryright (c) 1997 by D-Link Corporation. All rights reserved.
 *
 *  Revision History:
 *
 *  By          Date        Ver     Brief
 *  ----------- ----------- ------- ------------------------------------------
 *  Ken Vong    97-04-10    Beta    Create.
 *  David Hsieh 1997-July   Beta    Arrange.
 *
 */

#include "osdef.h"
#include "typesdef.h"

#include "pub.h"
#include "mii.h"
#include "mactophy.h"
#include "miifcn.h"
#include "c140kwd.h"

#include "debug.h"


/*----------------------------------------------------------------------------
                    Public Routines
-----------------------------------------------------------------------------*/

/*
 *  ConvertMediaOptionToMiiMediaType
 *
 *  Description:    Convert Media Option ( User select ) to media type.
 *
 *  Input:
 *
 *      tPPUB - pPub
 *      tPMEDIA_CTRL            Media Control Blk
 *      USHORT - wCType         connection type
 *
 *  Output: N/A
 *
 *  Return: N/A
 *
 *  Note:
 *
 */
static
tVOID
ConvertMediaOptToMiiMediaType(
    tPMEDIA_CTRL    pMediaCtrlBlk,
    tUSHORT         wCType
    )
{
    switch( wCType )
    {
        default:
        case CT_NWAY_AUTO:
            pMediaCtrlBlk->MiiMediaType.cControl = MII_NWAY_ENABLE |
                                                   MII_NWAY_AUTOSENSE;
            pMediaCtrlBlk->MiiMediaType.cConnection = MII_10BASE_T;
            break;
        case CT_100BASE_TX_FD:
            pMediaCtrlBlk->MiiMediaType.cControl = 0;
            pMediaCtrlBlk->MiiMediaType.cConnection = MII_100BASE_TX_FD;
            break;
        case CT_100BASE_TX:
            pMediaCtrlBlk->MiiMediaType.cControl = 0;
            pMediaCtrlBlk->MiiMediaType.cConnection = MII_100BASE_TX;
            break;
        case CT_10BASE_T_FD:
            pMediaCtrlBlk->MiiMediaType.cControl = 0;
            pMediaCtrlBlk->MiiMediaType.cConnection = MII_10BASE_T_FD;
            break;
        case CT_10BASE_T:
            pMediaCtrlBlk->MiiMediaType.cControl = 0;
            pMediaCtrlBlk->MiiMediaType.cConnection = MII_10BASE_T;
            break;
        case CT_NWAY_100BASE_TX_FD:
            pMediaCtrlBlk->MiiMediaType.cControl = MII_NWAY_ENABLE;
            pMediaCtrlBlk->MiiMediaType.cConnection = MII_100BASE_TX_FD;
            break;
        case CT_NWAY_100BASE_TX:
            pMediaCtrlBlk->MiiMediaType.cControl = MII_NWAY_ENABLE;
            pMediaCtrlBlk->MiiMediaType.cConnection = MII_100BASE_TX;
            break;
        case CT_NWAY_10BASE_T_FD:
            pMediaCtrlBlk->MiiMediaType.cControl = MII_NWAY_ENABLE;
            pMediaCtrlBlk->MiiMediaType.cConnection = MII_10BASE_T_FD;
            break;
        case CT_NWAY_10BASE_T:
            pMediaCtrlBlk->MiiMediaType.cControl = MII_NWAY_ENABLE;
            pMediaCtrlBlk->MiiMediaType.cConnection = MII_10BASE_T;
            break;
    }
}


/*
 *  bMacToPhyInit
 *
 *  Description:    1. search for the existing phy media.
 *                  2. get media capabilities.
 *
 *  Input:
 *
 *      tPPUB - pPub
 *      tPMEDIA_CTRL            Media Control Blk
 *      USHORT - wCType         connection type
 *
 *  Output:
 *
 *      Adapter->MediaCtrlBlk
 *
 *  Return:
 *
 *      eSUCCESS - if media found and successfully initialized.
 *      eFAIL    - if media not found or fail to initialize.
 *
 *  Note:   N/A
 *
 */
tRESULT
bMacToPhyInit(
    tPPUB           pPub,
    tPMEDIA_CTRL    pMediaCtrlBlk,
    tUSHORT         wCType
    )
{
    tUSHORT         wCapabilities;

    DBG_MII( DbgPrint("MacToPhyInit{,"); )

    /* Convert Media Option to MiiMediaType                         */
    /* MiiMediaType contains : 1. Nway mode (enabled/disabled).     */
    /*                         2. Current media setting.            */
    /* PS: The Current media setting is not valid now.              */
    ConvertMediaOptToMiiMediaType(pMediaCtrlBlk,wCType);
    /* Find and initialize the MII Phy */
    if ( bFindAndInitMiiPhy(pPub,&pMediaCtrlBlk->PhyInfo) == eFAIL )
    {
        DBG_MII( DbgPrint("FindAndInitMiiPhyFail}\n"); )
        return eFAIL;
    }

    /* Get MII Phy Capability   */
    wCapabilities = wGetMiiPhyCapabilities(pPub, &pMediaCtrlBlk->PhyInfo);

    DBG_MII( DbgPrint("PhyCap=0x%x,",wCapabilities); )

    pMediaCtrlBlk->PhyInfo.wLocalAdvertisement = 0;
    if (wCapabilities & MII_BMSR_100BASETX_FD)
    {
        pMediaCtrlBlk->PhyInfo.wLocalAdvertisement |= MII_ANAR_100BASETX_FD;
    }
    if (wCapabilities & MII_BMSR_100BASETX)
    {
        pMediaCtrlBlk->PhyInfo.wLocalAdvertisement |= MII_ANAR_100BASETX;
    }
    if (wCapabilities & MII_BMSR_10BASET_FD)
    {
        pMediaCtrlBlk->PhyInfo.wLocalAdvertisement |= MII_ANAR_10BASET_FD;
    }
    if (wCapabilities & MII_BMSR_10BASET)
    {
        pMediaCtrlBlk->PhyInfo.wLocalAdvertisement |= MII_ANAR_10BASET;
    }

    DBG_MII( DbgPrint("Advertisement=0x%x,ReturnSuccess}\n",pMediaCtrlBlk->PhyInfo.wLocalAdvertisement); )
    return eSUCCESS;
}


/*
 *  bMacSetPhyControl
 *
 *  Description:    1  Reset MII Phy
 *                  2. Restart the MIIPhy.
 *                  3. Update Media Conection Type.
 *
 *  Input:
 *
 *      tPPUB - pPub
 *      tPMEDIA_CTRL            Media Control Blk
 *
 *  Output: N/A
 *
 *  Return:
 *
 *      eSUCCESS - if complete successful.
 *      eFAIL    - if control mode unsupported.
 *
 *  Note:   N/A
 *
 */
tRESULT
bMacSetPhyControl(
    tPPUB           pPub,
    tPMEDIA_CTRL    pMediaCtrlBlk
    )
{
    /* Reset MII Phy   */
    MiiPhyAdminControl(
                    pPub,
                    &pMediaCtrlBlk->PhyInfo,
                    eADMIN_RESET
                    );

    /* Set Phy in operational mode */
    MiiPhyAdminControl(
                    pPub,
                    &pMediaCtrlBlk->PhyInfo,
                    eADMIN_OPERATIONAL
                    );

    /*  Restart MII Phy */
    if (bSetMiiPhyConnection(
                        pPub,
                        &pMediaCtrlBlk->PhyInfo,
                        pMediaCtrlBlk->MiiMediaType
                        ) == eFAIL )
    {
        return eFAIL;
    }

    return eSUCCESS;
}


/*
 *  bDetectMediaType
 *
 *  Description:    1. Get connection status.
 *                  2. Check if media mode is AutoSense.
 *                  3. If AutoSense, detect the active media. Switch to
 *                     the active media or default media (if link fail).
 *
 *  Input:
 *
 *      tPPUB - pPub
 *      tPMEDIA_CTRL            Media Control Blk
 *
 *  Output:
 *
 *      Adapter Link Status
 *      Adapter Current Media Type
 *
 *  Return:
 *
 *      eTRUE   - if link pass.
 *      eFALSE  - if link fail or fail to detect.
 *
 *  Note:
 *
 *      N/A
 *
 */
tBOOLEAN
bDetectMediaType(
    tPPUB           pPub,
    tPMEDIA_CTRL    pMediaCtrlBlk
    )
{
    if (bMacSetPhyControl(
                        pPub,
                        pMediaCtrlBlk
                        ) == eFAIL)
    {
        return eFALSE;
    }

    if (pMediaCtrlBlk->MiiMediaType.cControl & MII_NWAY_AUTOSENSE)
    {
        /* if we are in autosense mode, get the active media type. */
        return bGetMiiPhyActiveConnection(
                                        pPub,
                                        &pMediaCtrlBlk->PhyInfo,
                                        &pMediaCtrlBlk->MiiMediaType
                                        );
    }
    else
    {
        /* if we are in forced media mode, check link status only. */
        return bGetMiiPhyLinkStatus(
                                pPub,
                                &pMediaCtrlBlk->PhyInfo
                                );
    }
}


/*
 *  DynamicSenseMedia
 *
 *  Description:    1. Get link status and active connection type.
 *                  2. Compare with previous connection type.
 *                  3. Determines if MAC and PHY need to change setting.
 *
 *  Input:
 *
 *      pPub            - pointer to tPUB data structure.
 *      pMediaCtrlBlk   - pointer to tMEDIA_CTRL data structure.
 *      pLineSpeed100   - pointer to tBOOLEAN type variable.
 *      pFDEnable       - pointer to tBOOLEAN type variable.
 *
 *  Output:
 *
 *      If operation mode is autosense and return lin pass, pLineSpeed100 and
 *      pFDEnable will contains detected line speed and duplex mode status.
 *
 *  Return:
 *
 *      eTRUE   - if link pass.
 *      eFALSE  - if link fail or sense error.
 *
 *  Note:
 *
 *      The return value in pLineSpeed100 and pFDEnable will be valid only in
 *      line pass of autosene mode.
 *
 */
tBOOLEAN
DynamicSenseMedia(
    tPPUB           pPub,
    tPMEDIA_CTRL    pMediaCtrlBlk,
    tBOOLEAN        *pLineSpeed100,
    tBOOLEAN        *pFDEnable
    )
{
    tMII_MEDIA_TYPE         ConnectionType = pMediaCtrlBlk->MiiMediaType;
    if (pMediaCtrlBlk->MiiMediaType.cControl & MII_NWAY_AUTOSENSE)
    {
        if (bGetMiiPhyActiveConnection(
                                    pPub,
                                    &pMediaCtrlBlk->PhyInfo,
                                    &ConnectionType
                                    ) == eTRUE)
        { 
            if (pMediaCtrlBlk->MiiMediaType.cConnection !=
                ConnectionType.cConnection)
            {
                pMediaCtrlBlk->MiiMediaType.cConnection =
                                            ConnectionType.cConnection;
            }
            switch (ConnectionType.cConnection)
            {
                case MII_100BASE_TX_FD:
                    *pLineSpeed100 = eTRUE;
                    *pFDEnable = eTRUE;
                    break;
                case MII_100BASE_TX:
                    *pLineSpeed100 = eTRUE;
                    *pFDEnable = eFALSE;
                    break;
                case MII_10BASE_T_FD:
                    *pLineSpeed100 = eFALSE;
                    *pFDEnable = eTRUE;
                    break;
                case MII_10BASE_T:
                    *pLineSpeed100 = eFALSE;
                    *pFDEnable = eFALSE;
                    break;
            }
            return eTRUE;
        }
        else
        {
            return eFALSE;
        }
    }
    else
    {
        return bGetMiiPhyLinkStatus(pPub, &pMediaCtrlBlk->PhyInfo);
    }
}
