/*
 *  MIIPHY.c
 *
 *  Description:    This module supports the directly access to PHY registers.
 *                  And some basic control functions.
 *
 *  Copyright (c) 1997 by D-Link Corporation. All rights reserved.
 *
 *  Created by Ken Vong
 *
 *  Revision History:
 *
 *  By          Date        Ver     Brief
 *  ----------- ----------- ------- -----------------------------------------
 *  Ken Vong    97-04-10    Beta    Create.
 *  David Hsieh 1997-July   Beta    Arrange.
 *
 */
//#define DEBUG_MII
#include "osdef.h"
#include "typesdef.h"
#include "ioaccess.h"
#include "pub.h"
#include "mii.h"
#include "mactophy.h"
#include "dlkdelay.h"
#include "miifcn.h"
#include "debug.h"


#if OS_TYPE == LINUX
#include <linux/delay.h>
#include <asm/io.h>
#endif /*LINUX*/
 
#if OS_TYPE == ODI32
tULONG In32(tPVOID busTag, tPVOID ioAddr);
tVOID Out32(tPVOID BusTag, tPVOID ioAddr, tULONG outputVal);
#endif
#if OS_TYPE == DOS
tULONG inpd(tUSHORT wPort);
tVOID outpd(tUSHORT wPort, tULONG dwData);
#endif

/*-----------------------------------------------------------------------------
                      Static Routines
-----------------------------------------------------------------------------*/

/*
 *  Send
 *
 *  Description:    This routine write the specified value to MiiPort.
 *
 *  Inputs:
 *
 *      pPub    - pointer to tPUB data structure.
 *      Port    - Mii port.
 *      dwData  - Value to write.
 *
 */
static
tVOID
MiiSend(
    tPPUB       pPub,
    tPORT       Port,
    tULONG      dwData
    )
{
    mOutD(Port, dwData);
    Delay(MII_DELAY);
}



/*
 *  MiiSend_0
 *
 *  Description:    This routine write 0 to MDIO.
 *
 */
static
tVOID
MiiSend_0(
    tPPUB       pPub,
    tPORT       Port
    )
{
    MiiSend(pPub, Port, MII_WRITE|        MII_DATA_0);
    MiiSend(pPub, Port, MII_WRITE|MII_CLK|MII_DATA_0);
}



/*
 *  MiiSend_1
 *
 *  Description:    This routine write 1 to MDIO.
 *
 */
static
tVOID
MiiSend_1(
    tPPUB       pPub,
    tPORT       Port
    )
{
    MiiSend(pPub, Port, MII_WRITE|        MII_DATA_1);
    MiiSend(pPub, Port, MII_WRITE|MII_CLK|MII_DATA_1);
}



/*
 *  SendBitsToPhy
 *
 *  Description:    Tihs routine write bits serially to MDIO.
 *
 */
static
tVOID
SendBitsToPhy(
    tPPUB       pPub,
    tPORT       Port,
    tINT        nBitsToSend,
    tULONG      dwPattern
    )
{
    tINT        nIdx;

    for (nIdx=nBitsToSend; nIdx; nIdx--)
    {
        if (dwPattern & 0x80000000)
            MiiSend_1(pPub, Port);
        else
            MiiSend_0(pPub, Port);
        dwPattern = dwPattern << 1;
    }
}



/*
 *  SendThreeState
 *
 *  Description:    This routine will write MDIO as three state.
 *
 */
static
tVOID
SendThreeState(
    tPPUB       pPub,
    tPORT       Port
    )
{
    MiiSend(pPub, Port, MII_WRITE_THREE_STATE|        MII_DATA_0);
    MiiSend(pPub, Port, MII_WRITE_THREE_STATE|MII_CLK|MII_DATA_0);
}


/*
 *  SendEnd
 *
 */
static
tVOID
SendEnd(
    tPPUB       pPub,
    tPORT       Port
    )
{
    MiiSend(pPub, Port, MII_WRITE_THREE_STATE);
}



/*
 *  bMiiPhyReadRegister
 *
 *  Description:    Read the contents of PHY register.
 *
 *  Input:
 *
 *      pPub
 *      pPhyInfoPtr - the MiiPhyInfo block
 *      wRegNum - register number to read.
 *      pRegDat - pointer to user specified space.
 *
 *  Output:
 *
 *      *pRegDat    Content of target register.
 *
 *  Return:
 *
 *      eSUCCESS - if read successfully.
 *      eFAIL    - if fail to read.
 *
 *  Note:
 *
 *
 */
tRESULT
bMiiPhyReadRegister(
    tPPUB           pPub,
    tPMII_PHY_INFO  pPhyInfo,
    tUSHORT         wRegNum,
    tPUSHORT        pRegDat
    )
{
    tULONG          dwCmdWord;
    tULONG          dwData;
    tUSHORT         wTemp;
    tINT            nIdx;
    tRESULT         bRetVal;
    tPORT           Port = pPub->Port;

    /* Write preamble */
    SendBitsToPhy(pPub, Port, 32, MII_PRE);

    /* Prepare command word */
    /* ST(2), OP(2), ADDR(5), REG#(5), TA(2), Data(16) total 32 bits */
    dwCmdWord = (tULONG)pPhyInfo->wMIIPhyAddress << PHY_ADDR_ALIGN;
    dwCmdWord |= ((tULONG)wRegNum << REG_ADDR_ALIGN);
    dwCmdWord |= MII_READ_FRAME;
    SendBitsToPhy(pPub, pPub->Port, 14, dwCmdWord);

    /*  Turnaround & 0 */
    SendThreeState(pPub, Port);
    mInD(Port, &dwData);

    /* Check if read success */
    if ((dwData >> MII_MDI_BITS_SHIFT) & 1)
    {
        bRetVal = eFAIL;
    }
    else
    {
        bRetVal = eSUCCESS;
    }

    /* Read target word */
    *pRegDat = 0;
    for (nIdx=0; nIdx<16; nIdx++)
    {
        mOutD(Port, MII_READ        );
        Delay(MII_DELAY);
        mOutD(Port, MII_READ|MII_CLK);
        Delay(MII_DELAY);
        mInD(Port, &dwData);

        wTemp = (tUSHORT)((dwData >> MII_MDI_BITS_SHIFT) & 1);
        (*pRegDat) = ((*pRegDat) << 1) | wTemp;
    }

    SendThreeState(pPub, Port);
    SendThreeState(pPub, Port);
    SendThreeState(pPub, Port);

    SendEnd(pPub, Port);
    return bRetVal;
}


/*
 *  MiiPhyWriteRegister
 *
 *  Description:    1. Write contents to register.
 *
 *  Input:
 *
 *      pPub
 *      pPhyInfo -   the MiiPhyInfo block
 *      wRegNum -       number of target register
 *      wRegDat -       Content to write
 *
 *  Output:
 *
 *      N/A
 *
 *  Return:
 *
 *      N/A
 *
 *  Note:
 *
 *      N/A
 *
 */
tVOID
MiiPhyWriteRegister(
    tPPUB           pPub,
    tPMII_PHY_INFO  pPhyInfo,
    tUSHORT         wRegNum,
    tUSHORT         wRegDat
    )
{
    tULONG          dwCmdWord;

    /* Write preamble */
    SendBitsToPhy(pPub, pPub->Port, 32, MII_PRE);

    /* Prepare command word */
    /* ST(2), OP(2), ADDR(5), REG#(5), TA(2), Data(16) total 32 bits */
    dwCmdWord = (tULONG)pPhyInfo->wMIIPhyAddress << PHY_ADDR_ALIGN;
    dwCmdWord |= ((tULONG)wRegNum << REG_ADDR_ALIGN);
    dwCmdWord |= MII_WRITE_FRAME;
    dwCmdWord |= (tULONG)wRegDat;
    SendBitsToPhy(pPub, pPub->Port, 32, dwCmdWord);

    SendThreeState(pPub, pPub->Port);
    SendThreeState(pPub, pPub->Port);
    SendThreeState(pPub, pPub->Port);

    SendEnd(pPub, pPub->Port);
    return;
}

/*-----------------------------------------------------------------------------
                      Public Routines
-----------------------------------------------------------------------------*/


/*
 *  bFindAndInitMiiPhy
 *
 *  Description:    1. Search MiiPhy device.
 *                  2. Get Phy ID and all register contents.
 *
 *  Input:
 *
 *      tPPUB - pPub
 *      tPMII_PHY_INFO - pPhyInfo the MiiPhyInfo data structure.
 *
 *  Output:
 *
 *      PhyInfo->bStructValid
 *      PhyInfo->wPhyAddress
 *      PhyInfo->dwPhyID
 *
 *  Return:
 *
 *      tRESULT     eSUCCESS - if found and successful initialized.
 *                  eFAIL    - if not found or fail to initialize.
 *
 */
tRESULT
bFindAndInitMiiPhy(
    tPPUB           pPub,
    tPMII_PHY_INFO  pPhyInfo
    )
{
    tUSHORT         wPhyAdd;
    tUINT           nIndex;
    tUSHORT         wRegVal;

    DBG_MII( DbgPrint("FindAndInitMiiPhy{,"); )
    DBG_MII( DbgPrint("Port--[%#4.4x] ",pPub->Port);) 
    /*  Search  Phy Address */
    for ( wPhyAdd=0; wPhyAdd<=MAX_PHY_ADDR; wPhyAdd++ )
    {
        pPhyInfo->wMIIPhyAddress = wPhyAdd;
        DBG_MII( DbgPrint("PhyAddr=0x%x,",wPhyAdd); )
        if (bMiiPhyReadRegister(
                            pPub,
                            pPhyInfo,
                            MII_BMSR,
                            &wRegVal
                            ) == eSUCCESS)
        {
            /* Chech if we really found the PHY. */
            if (wRegVal == 0) continue;

            DBG_MII( DbgPrint("PhyFound,"); )
            /*  Save ALL PHY Register   */
            for ( nIndex=0; nIndex<=MAX_PHY_REGS; nIndex++ )
            {
                bMiiPhyReadRegister(
                                pPub,
                                pPhyInfo,
                                (tUSHORT)nIndex,
                                &wRegVal
                                );
                pPhyInfo->wPhyRegs[nIndex] = wRegVal;
                DBG_MII( DbgPrint("%d:0x%x,",nIndex,wRegVal); )
            }

            /*  Save Phy ID */
            /*  Phy ID 2 ( bit 0-4 )  Four Bits of Vendor Model revision */
            pPhyInfo->dwPhyID =
                            ((tULONG)pPhyInfo->wPhyRegs[MII_PHY_ID1] << 16)
                             | pPhyInfo->wPhyRegs[MII_PHY_ID2];
            pPhyInfo->bStructValid = eTRUE;

            return( eSUCCESS );
        }
    }
    DBG_MII( DbgPrint("ReturnFail}\n"); )
    return ( eFAIL );
}


/*
 *  SetMiiPhyAdvertisement
 *
 *  Description:    1. Set MII Phy Advertisement
 *
 *  Input:
 *
 *      tPPUB
 *      tPMII_PHY_INFO - pPhyInfo    pointer to MiiPhyInfo bloack.
 *      wRegVal                         content to write
 *
 *
 *  Output: N/A
 *
 *  Return: N/A
 *
 */
tVOID
SetMiiPhyAdvertisement(
    tPPUB           pPub,
    tPMII_PHY_INFO  pPhyInfo,
    tUSHORT         wRegVal
    )
{
    pPhyInfo->wPhyRegs[MII_ANAR] = wRegVal;
    MiiPhyWriteRegister(
                    pPub,
                    pPhyInfo,
                    (tUSHORT)MII_ANAR,
                    (tUSHORT)(wRegVal | MII_ANAR_SELECTOR)
                    );
    return;
}



/*
 *  wGetMiiPhyCapabilities
 *
 *  Description:    1. Return the read Capabilities
 *
 *  Input:
 *
 *      tPPUB
 *      tPMII_PHY_INFO - pPhyInfo     pointer to MiiPhy info block.
 *
 *  Output: N/A
 *
 *  Return:
 *
 *      (USHORT) MiiPhy Capabilities
 *
 *  Note:
 *
 *      N/A
 *
 */
tUSHORT
wGetMiiPhyCapabilities(
    tPPUB           pPub,
    tPMII_PHY_INFO  pPhyInfo
    )
{
    /* Just return the bits about capability */
    return (tUSHORT)(pPhyInfo->wPhyRegs[MII_BMSR] & (MII_BMSR_100BASETX_FD |
                                                     MII_BMSR_100BASETX    |
                                                     MII_BMSR_10BASET_FD   |
                                                     MII_BMSR_10BASET));
}


/*
 *  MiiPhyAdminControl
 *
 *  Description:    1. Perform the control command on the specified Phy.
 *                  2. Save the control command on MiiPhyInfo block.
 *
 *  Input:
 *
 *      tPPUB - pPub
 *      tPMII_PHY_INFO - pPhyInfo    pointer to MiiPhyInfo bloack.
 *      tMII_CONTROL - Control          Control status
 *
 *  Output: N/A
 *
 *  Return: N/A
 *
 */
tVOID
MiiPhyAdminControl(
    tPPUB           pPub,
    tPMII_PHY_INFO  pPhyInfo,
    tMII_STATUS     Control
    )
{
    tINT            nIdx = 0;
    tMII_STATUS     Status;

    /* Clear previous control bits */
    pPhyInfo->wPhyRegs[MII_BMCR] &= ~(MII_BMCR_RESET |
                                      MII_BMCR_POWER_DOWN |
                                      MII_BMCR_ISOLATE);

    switch(Control)
    {
        case eADMIN_RESET:

            MiiPhyWriteRegister(
                    pPub,
                    pPhyInfo,
                    (tUSHORT)MII_BMCR,
                    (tUSHORT)(pPhyInfo->wPhyRegs[MII_BMCR] | MII_BMCR_RESET)
                    );
            while ((nIdx++) < MII_RESET_TIME_OUT)
            {
                MiiPhyAdminStatus(
                            pPub,
                            pPhyInfo,
                            &Status
                            );
                if (Status != eADMIN_RESET) break;
            }
            break;

        case eADMIN_POWER_DOWN:

            MiiPhyWriteRegister(
                pPub,
                pPhyInfo,
                (tUSHORT)MII_BMCR,
                (tUSHORT)(pPhyInfo->wPhyRegs[MII_BMCR] | MII_BMCR_POWER_DOWN)
                );
            break;

        case eADMIN_ISOLATE:

            MiiPhyWriteRegister(
                pPub,
                pPhyInfo,
                (tUSHORT)MII_BMCR,
                (tUSHORT)(pPhyInfo->wPhyRegs[MII_BMCR] | MII_BMCR_ISOLATE)
                );
            break;

        case eADMIN_OPERATIONAL:

            MiiPhyWriteRegister(
                            pPub,
                            pPhyInfo,
                            MII_BMCR,
                            pPhyInfo->wPhyRegs[MII_BMCR]
                            );
            break;

        default:
            /* Not Implement yet    */
            break;
    }
    return;
}



/*
 *  MiiPhyAdminStatus
 *
 *  Description:    1. Returns PHY admin status
 *                  2. Refresh the MiiPhyInfo block setting of Control command.
 *
 *  Input:
 *
 *      tPPUB - pPub
 *      tPMII_PHY_INFO - pPhyInfo
 *      tPMII_STATUS - pStatus
 *
 *  Output:
 *  Return:
 *  Note:
 *
 *
 */
tVOID
MiiPhyAdminStatus(
    tPPUB           pPub,
    tPMII_PHY_INFO  pPhyInfo,
    tPMII_STATUS    pStatus
    )
{
    tINT            nCount = 3;
    tUSHORT         wRegVal;

    while (nCount--)
    {
        bMiiPhyReadRegister(
                        pPub,
                        pPhyInfo,
                        MII_BMCR,
                        &wRegVal
                        );
        if (wRegVal) break;
    }

    pPhyInfo->wPhyRegs[MII_BMCR] = wRegVal;

    if (wRegVal & MII_BMCR_RESET)
    {
        *pStatus = eADMIN_RESET;
    }
    else if (wRegVal & MII_BMCR_LOOP_BACK)
    {
        *pStatus = eADMIN_LOOP_BACK;
    }
    else if (wRegVal & MII_BMCR_POWER_DOWN)
    {
        *pStatus = eADMIN_POWER_DOWN;
    }
    else if (wRegVal & MII_BMCR_ISOLATE)
    {
        *pStatus = eADMIN_ISOLATE;
    }
    else
    {
        *pStatus = eADMIN_OPERATIONAL;
    }
    return;
}



/*
 *  bSetMiiPhyConnection
 *
 *  Description:    This routine will set BMCR to the target operation mode
 *                  according to the MiiMediaType. If media type is Nway
 *                  enabled and forced media, the ANAR will be set to advertise
 *                  only one media which is set in MiiMediaType. If media type
 *                  is Nway Autosense, the ANAR will be set to advertise the
 *                  medium which are described in
 *                  PhyInfo->wLocalAdvertisement.
 *
 *  Input:
 *
 *      pPub            - pointer to tPUB data structure.
 *      pPhyInfo        - pointer to tMII_PHY_INFO data structure.
 *      MiiMediaType    - tMII_MEDIA_TYPE.
 *
 *  Output:
 *
 *      N/A
 *
 *  Return:
 *
 *      tRESULT     eSUCCESS - media set successfully.
 *                  eFAIL    - media set fail or media not supported.
 *
 *  Note:
 *
 */
tRESULT
bSetMiiPhyConnection(
    tPPUB               pPub,
    tPMII_PHY_INFO      pPhyInfo,
    tMII_MEDIA_TYPE     MiiMediaType
    )
{
    tINT                nI;

    if (MiiMediaType.cControl & MII_NWAY_AUTOSENSE)
    {
        /* if NWAY_AUTOSENSE enabled, Nway must be enabled. */
        SetMiiPhyAdvertisement(
                            pPub,
                            pPhyInfo,
                            pPhyInfo->wLocalAdvertisement
                            );
    }
    else if (MiiMediaType.cControl & MII_NWAY_ENABLE)
    {

        /* Nway forced media */
        switch( MiiMediaType.cConnection )
        {
            case MII_100BASE_TX_FD:
                SetMiiPhyAdvertisement(
                                    pPub,
                                    pPhyInfo,
                                    MII_ANAR_100BASETX_FD
                                    );
                break;
            case MII_100BASE_TX:
                SetMiiPhyAdvertisement(
                                    pPub,
                                    pPhyInfo,
                                    MII_ANAR_100BASETX
                                    );
                break;
            case MII_10BASE_T_FD:
                SetMiiPhyAdvertisement(
                                    pPub,
                                    pPhyInfo,
                                    MII_ANAR_10BASET_FD
                                    );
                break;
            case MII_10BASE_T:
                SetMiiPhyAdvertisement(
                                    pPub,
                                    pPhyInfo,
                                    MII_ANAR_10BASET
                                    );
                break;
            default:
                /* Unsupported media type found! */
                return eFAIL;
        }
    }
    else
    {
        /*  Nway Disable */
        switch( MiiMediaType.cConnection )
        {
            case MII_100BASE_TX_FD:
                MiiPhyWriteRegister(
                                pPub,
                                pPhyInfo,
                                MII_BMCR,
                                MII_BMCR_SPEED100 | MII_BMCR_DUPLEX_MODE
                                );
                Delay(MII_DELAY);
                break;
            case MII_100BASE_TX:
                MiiPhyWriteRegister(
                                pPub,
                                pPhyInfo,
                                MII_BMCR,
                                MII_BMCR_SPEED100
                                );
                Delay(MII_DELAY);
                break;
            case MII_10BASE_T_FD:
                MiiPhyWriteRegister(
                                pPub,
                                pPhyInfo,
                                MII_BMCR,
                                MII_BMCR_DUPLEX_MODE
                                );
                Delay(MII_DELAY);
                break;
            case MII_10BASE_T:
                MiiPhyWriteRegister(
                                pPub,
                                pPhyInfo,
                                MII_BMCR,
                                MII_BMCR_SPEED10
                                );
                Delay(MII_DELAY);
                break;
            default:
                /* Unsupported media found,  return fail! */
                return eFAIL;
        }
        return eSUCCESS;
    }

    /* Set Nway enabled and wait for Nway complete */

    MiiPhyWriteRegister(
                    pPub,
                    pPhyInfo,
                    MII_BMCR,
                    MII_BMCR_NWAY_ENABLE | MII_BMCR_RESTART_NWAY
                    );

    /*  3000ms = 3sec   */
    for ( nI=0; nI<3000; nI++)
    {
        Delay(MILLISECOND);
        bMiiPhyReadRegister(
                        pPub,
                        pPhyInfo,
                        MII_BMSR,
                        &pPhyInfo->wPhyRegs[MII_BMSR]
                        );
        if ( pPhyInfo->wPhyRegs[MII_BMSR] & MII_BMSR_NWAY_COMPLETE )
        {
            break;
        }
    }

    return eSUCCESS;
}



/*
 *  bGetMiiPhyActiveConnection
 *
 *  Description:    This routine is used to get the active media type.
 *                  It should be called to get the current media type in
 *                  AUTOSENSE mode. And it should not be called in forced
 *                  media mode.
 *
 *  Input:
 *
 *      tPPUB - pPub
 *      pPhyInfo - the MiiPhyInfo block
 *      pConnection - pointer of user specified space.
 *
 *  Output:
 *
 *      *pConnection = (MII_MEDIA_TYPE)
 *
 *  Return:
 *
 *      eTRUE   - if link pass,
 *      eFALSE  - if link fail or fail to get connection.
 *
 *  Note:
 *
 *      N/A
 *
 */
tBOOLEAN
bGetMiiPhyActiveConnection(
    tPPUB               pPub,
    tPMII_PHY_INFO      pPhyInfo,
    tPMII_MEDIA_TYPE    pMediaType
    )
{
    tUSHORT             wRegVal;    /* temp */
    tINT                nIdx;

    if (pMediaType->cControl & MII_NWAY_AUTOSENSE)
    {
        /* Make sure the Nway is completed. */
        for (nIdx=0; nIdx<3000; nIdx++)
        {
            Delay(MILLISECOND);
            bMiiPhyReadRegister(
                            pPub,
                            pPhyInfo,
                            MII_BMSR,
                            &pPhyInfo->wPhyRegs[MII_BMSR]
                            );
            if (pPhyInfo->wPhyRegs[MII_BMSR] & MII_BMSR_NWAY_COMPLETE)
            {
                break;
            }
        }
    }
    else
    {
        return eFALSE;
    }

    bMiiPhyReadRegister(
                    pPub,
                    pPhyInfo,
                    MII_BMSR,
                    &pPhyInfo->wPhyRegs[MII_BMSR]
                    );
    /* check MII Phy is Link OK */
    if ( pPhyInfo->wPhyRegs[MII_BMSR] & MII_BMSR_LINK )
    {
        bMiiPhyReadRegister(
                        pPub,
                        pPhyInfo,
                        MII_ANAR,
                        &pPhyInfo->wPhyRegs[MII_ANAR]
                        );
        bMiiPhyReadRegister(
                        pPub,
                        pPhyInfo,
                        MII_ANLPAR,
                        &pPhyInfo->wPhyRegs[MII_ANLPAR]
                        );
        wRegVal = ( pPhyInfo->wPhyRegs[MII_ANAR] &
                    pPhyInfo->wPhyRegs[MII_ANLPAR] &
                    (MII_ANAR_100BASETX_FD | MII_ANAR_100BASETX |
                     MII_ANAR_10BASET_FD   | MII_ANAR_10BASET)
                  );

        if ( wRegVal & MII_ANAR_100BASETX_FD )
        {
            pMediaType->cConnection = MII_100BASE_TX_FD;
        }
        else if ( wRegVal & MII_ANAR_100BASETX )
        {
            pMediaType->cConnection = MII_100BASE_TX;
        }
        else if ( wRegVal & MII_ANAR_10BASET_FD )
        {
            pMediaType->cConnection = MII_10BASE_T_FD;
        }
        else if ( wRegVal & MII_ANAR_10BASET )
        {
            pMediaType->cConnection = MII_10BASE_T;
        }
        return eTRUE;
    }
    else
    {
        return eFALSE;
    }
}


/*
 *  tGetMiiPhyLinkStatus
 *
 *  Description:    1. Read BMSR for link status, and return.
 *
 *  Input:
 *
 *      tPPUB - pPub
 *      pPhyInfo - the MiiPhyInfo block
 *
 *  Output:
 *
 *      N/A
 *
 *  Return:
 *
 *      tBOOLEAN    eTRUE   if link pass
 *                  eFALSE  if link fail
 *
 *  Note:
 *
 *      N/A
 *
 */
tBOOLEAN
bGetMiiPhyLinkStatus(
    tPPUB           pPub,
    tPMII_PHY_INFO  pPhyInfo
    )
{

    bMiiPhyReadRegister(
                    pPub,
                    pPhyInfo,
                    MII_BMSR,
                    &pPhyInfo->wPhyRegs[MII_BMSR]
                    );
    if ( pPhyInfo->wPhyRegs[MII_BMSR] & MII_BMSR_LINK )
    {
        return ( eTRUE );
    }
    else
    {
        return( eFALSE );
    }
}
