/*
 *  SROM.c
 *
 *  Description:    This module supports all routines used for read, write
 *                  and parse serial ROM.
 *
 *  Copyright (c) 1997 by D-Link Corporation. All rights reserved.
 *
 *  Created by David Hsieh.
 *
 *  Revision History:
 *
 *  Name        Date        Ver     Brief
 *  ----------- ----------- ------- -----------------------------------------
 *  David Hsieh 1997-July   Beta    Create Chip140 driver.
 *
 */
#if OS_TYPE == LINUX
#include <linux/delay.h>
#include <asm/io.h>
#endif


#include "osdef.h"
#include "typesdef.h"
#include "ioaccess.h"
#include "pub.h"
#include "srom.h"
#include "dlkdelay.h"

#ifndef WRITE_EEPROM
#define WRITE_EEPROM    0
#endif

static tINT    nMaxCycles = SROM46_MAX_CYCLES;
static tUSHORT wOpAddrCnt = SROM46_OP_ADDRCNT;
static tUSHORT wOpRead    = SROM46_OP_READ;

#if WRITE_EEPROM
static tUSHORT wOpWrite   = SROM46_OP_WRITE;
static tUSHORT wOpErase   = SROM46_OP_ERASE;
static tUSHORT wOpEWEN    = SROM46_OP_EWEN;
static tUSHORT wOpEWDS    = SROM46_OP_EWDS;
#endif

#if OS_TYPE == ODI32
tULONG In32(tPVOID busTag, tPVOID ioAddr);
tVOID Out32(tPVOID busTag, tPVOID ioAddr, tULONG outputVal);
#endif




/*
 *  Send
 *
 *  Description:    This routine will out a value to EEPROM r/w port.
 *
 *  Inputs:
 *
 *      pPub            - pointer to tPUB structure.
 *      Port            - Read/Write EEPROM port.
 *      dwData          - Data to write.
 *
 *  Outputs;
 *  Returns:
 *
 *      N/A
 *
 */
static
tVOID
Send(
    tPPUB       pPub,
    tPORT       Port,
    tULONG      dwData
    )
{
    mOutD(Port, dwData);
    Delay(SROM_DELAY);
}


/*
 *  Send_0
 *
 *  Description:    This routine will send 0 in data out pin.
 *                  SK high time -> Min: 500 ns.
 *                  SK low time  -> Min: 500 ns.
 *
 *              CS      .--------
 *              CLK     .__----__
 *              DI      .________
 *
 *  Inputs:
 *
 *      pPub            - pointer to tPUB structure.
 *      Port            - Read/Write EEPROM port
 *
 *  Outputs:
 *  Returns:
 *
 *      N/A
 *
 */
static
tVOID
Send_0(
    tPPUB       pPub,
    tPORT       Port
    )
{
    Send(pPub, Port, CSR_WRITE|SEL_SROM|CS|    DATA_0);
    Send(pPub, Port, CSR_WRITE|SEL_SROM|CS|CLK|DATA_0);
    Send(pPub, Port, CSR_WRITE|SEL_SROM|CS|CLK|DATA_0);
    Send(pPub, Port, CSR_WRITE|SEL_SROM|CS|    DATA_0);
}



/*
 *  Send_1
 *
 *  Description:    This routine will send 1 in data out pin.
 *                  SK high time -> Min: 500 ns.
 *                  SK low time  -> Min: 500 ns.
 *
 *              CS      .--------
 *              CLK     .__----__
 *              DI      .--------
 *
 *  Inputs:
 *
 *      pPub            - pointer to tPUB structure.
 *      Port            - Read/Write EEPROM port
 *
 *  Outputs:
 *  Returns:
 *
 *      N/A
 *
 */
static
tVOID
Send_1(
    tPPUB       pPub,
    tPORT       Port
    )
{
    Send(pPub, Port, CSR_WRITE|SEL_SROM|CS|    DATA_1);
    Send(pPub, Port, CSR_WRITE|SEL_SROM|CS|CLK|DATA_1);
    Send(pPub, Port, CSR_WRITE|SEL_SROM|CS|CLK|DATA_1);
    Send(pPub, Port, CSR_WRITE|SEL_SROM|CS|    DATA_1);
}



/*
 *  ResetSendState
 *
 *  Description:    This routine reset CS.
 *                  CS hold time -> Min 0 ns.
 *
 *  Inputs:
 *
 *      pAdapter        - pointer to ADAPTER structure.
 *      Port            - Read/Write EEPROM port
 *
 *  Outputs:
 *  Returns:
 *
 *      N/A
 *
 */
static
tVOID
ResetSendState(
    tPPUB       pPub,
    tPORT       Port
    )
{
    mOutD(Port, (tULONG)CSR_WRITE|SEL_SROM|CS    );
    Delay(SROM_DELAY);
    mOutD(Port, (tULONG)CSR_WRITE|SEL_SROM       );
}


/*
 *  SetSendState
 *
 *  Description:    Set the CS signal.
 *                  CS setup time -> Min: 100 ns.
 *
 *              CS      .__--
 *              CLK     .____
 *              DI      .____
 *
 *  Inputs:
 *
 *      pPub            - pointer to tPUB structure.
 *      Port            - Read/Write EEPROM port.
 *
 *  Outputs:
 *  Returns:
 *
 *      N/A
 *
 */
static
tVOID
SetSendState(
    tPPUB       pPub,
    tPORT       Port
    )
{
    mOutD(Port, (tULONG)CSR_WRITE|SEL_SROM   );
    Delay(SROM_DELAY);
    mOutD(Port, (tULONG)CSR_WRITE|SEL_SROM|CS);
    Delay(SROM_DELAY);
}



/*
 *  SendBitsToChip
 *
 *  Description:    Send n bits to data out.
 *
 *  Inputs:
 *
 *      pPub            - pointer to tPUB structure.
 *      Port            - Read/Write EEPROM port
 *      nBitsToSend     - Number of bits to send
 *      wPattern        - bit pattern
 *
 *  Outputs:
 *  Returns:
 *
 *      N/A
 *
 */
static
tVOID
SendBitsToChip(
    tPPUB       pPub,
    tPORT       Port,
    tINT        nBitsToSend,
    tUSHORT     wPattern
    )
{
    tINT        nIdx;

    wPattern = (wPattern << (sizeof(tUSHORT)*8-nBitsToSend));
    for (nIdx=nBitsToSend; nIdx; nIdx--)
    {
        if (wPattern & 0x8000)
            Send_1(pPub, Port);
        else
            Send_0(pPub, Port);
        wPattern = wPattern << 1;
    }
}




/*
 *  bReadAWord
 *
 *  Description:    This routine will read a word from EEPRO.
 *
 *  Inputs:
 *
 *      pPub            - pointer to tPUB data structure.
 *      Port            - Read/Write EEPROM port.
 *      wIndex          - word index.
 *
 *  Outputs:
 *
 *      pData           - pointer to preallocated word width storage.
 *
 *  Returns:
 *
 *      eSUCCESS/eFAIL
 *
 */
tRESULT
bReadAWord(
    tPPUB               pPub,
    tPORT               Port,
    tUSHORT             wIndex,
    tPUSHORT            pData
    )
{
    tUSHORT     wPattern = (wIndex | wOpRead);
    tULONG      dwDout = 0;
    tINT        nIdx;

    SetSendState(pPub, Port);
    SendBitsToChip(pPub, Port, wOpAddrCnt, wPattern);
    mInD(Port, &dwDout);
    if (dwDout & SROM_DO_BIT_MASK)
    {
        return eFAIL;
    }

    *pData = 0;

    for (nIdx=0; nIdx<(sizeof(tUSHORT)*8); nIdx++)
    {
        *pData = *pData << 1;
        Send_0(pPub, Port);
        mInD(Port, &dwDout);
        if (dwDout & SROM_DO_BIT_MASK)
            *pData |= 1;
    }
    ResetSendState(pPub, Port);
    return eSUCCESS;
}




/*
 *  MakeSureSROMIsIdle
 *
 *  Description:    Makes sure the SROM is in idle state, by delivering it
 *                  enough clocks with CS set, Din=0. That way we make sure
 *                  the SROM isn't in the middle of instruction execution.
 *
 *  Inputs:
 *
 *      pAdapter        - Pointer to ADAPTER structure.
 *      Port           - Read/Write EEPROM port.
 *
 *  Outputs:
 *  Returns:
 *
 *      N/A
 *
 */
tVOID
MakeSureSROMIsIdle(
    tPPUB           pPub,
    tPORT           Port
    )
{
    tINT            nIdx;

    SetSendState(pPub, Port);
    for (nIdx=0; nIdx<nMaxCycles; nIdx++)
    {
        Send_0(pPub, Port);
    }
    ResetSendState(pPub, Port);
}


#if WRITE_EEPROM

/*
 *  WaitBusy
 *
 *  Description:    After WRITE/ERASE action, wait for READY signal from
 *                  EEPROM data out pin. (EEPROM r/w port bit 0).
 *
 *  Inputs:
 *
 *      pPub            - pointer to the tPUB structure.
 *      Port            - Read/Write EEPROM port.
 *
 *  Outputs:
 *  Returns:
 *      N/A
 *
 *  Note:
 *
 *      tWP Write Cycle Time : Max=10ms.
 *
 */
static
tVOID
WaitBusy(
    tPPUB           pPub,
    tPORT           Port
    )
{
    tUINT           nCount=500;
    tULONG          dwDout;

    while (nCount)
    {
        Delay(2);
        mInD(Port,&dwDout);
        if (dwDout & SROM_DO_BIT_MASK) break;
        nCount--;
    }
}



/*
 *  WriteWait
 *
 *  Description:    After WRITE/ERASE action, wait for READY signal from
 *                  EEPROM data out pin. (EEPROM r/w port bit 0).
 *
 *  Inputs:
 *
 *      pPub            - pointer to tPUB datat structure.
 *      Port            - Read/Write EEPROM port.
 *
 *  Outputs:
 *  Returns:
 *
 *      N/A
 *
 */
static
tVOID
WriteWait(
    tPPUB       pPub,
    tPORT       Port
    )
{
    SetSendState(pPub, Port);
    WaitBusy(pPub, Port);
    ResetSendState(pPub, Port);
}



/*
 *  WriteEnable
 *
 *  Description:    This routine will set EEPROM Erase/Write enable.
 *
 *  Inputs:
 *
 *      pPub            - pointer to tPUB structure.
 *      Port            - Read/Write EEPROM port.
 *
 *  Outputs:
 *  Inputs:
 *
 *      N/A
 *
 */
static
tVOID
WriteEnable(
    tPPUB       pPub,
    tPORT       Port
    )
{
    SetSendState(pPub, Port);
    SendBitsToChip(pPub, Port, wOpAddrCnt, wOpEWEN);
    ResetSendState(pPub, Port);
}



/*
 *  WriteDisable
 *
 *  Description:    This routine will set EEPROM Erase/Write disable.
 *
 *  Inputs:
 *
 *      pPub            - pointer to tPUB data structure.
 *      Port            - Read/Write EEPROM port.
 *
 *  Outputs:
 *  Returns:
 *
 *      N/A
 *
 */
static
tVOID
WriteDisable(
    tPPUB       pPub,
    tPORT       Port
    )
{
    SetSendState(pPub, Port);
    SendBitsToChip(pPub, Port, wOpAddrCnt, wOpEWDS);
    ResetSendState(pPub, Port);
}


/*
 *  Erase
 *
 *  Description:    This routine will erase one word.
 *
 *  Inputs:
 *
 *      pPub            - pointer to tPUB data structure.
 *      Port            - Read/Write EEPROM port.
 *      wIndex          - Index of target word.
 *
 *  Outputs:
 *  Returns:
 *
 *      N/A
 *
 */
static
tVOID
Erase(
    tPPUB       pPub,
    tPORT       Port,
    tUSHORT     wIndex
    )
{
    tUSHORT     wPattern = wIndex | wOpErase;

    SetSendState(pPub, Port);
    SendBitsToChip(pPub, Port, wOpAddrCnt, wPattern);
    ResetSendState(pPub, Port);
    WriteWait(pPub, Port);
}


/*
 *  WriteWordToEEPROM
 *
 *  Description:    This routine will send write command to serial EEPROM.
 *
 *  Inputs:
 *
 *      pPub            - pointer to tPUB data structure.
 *      Port            - Read/Write EEPROM port.
 *      wIndex          - Word index.
 *      wData           - Word data.
 *
 *  Outputs:
 *  Returns:
 *
 *      N/A
 *
 */
static
tVOID
WriteWordToEEPROM(
    tPPUB       pPub,
    tPORT       Port,
    tUSHORT     wIndex,
    tUSHORT     wData
    )
{
    USHORT      wPattern = wIndex | wOpWrite;

    SetSendState(pPub, Port);
    SendBitsToChip(pPub, Port, wOpAddrCnt, wPattern);
    SendBitsToChip(pPub, Port, sizeof(tUSHORT)*8, wData);
    ResetSendState(pPub, Port);
    WriteWait(pPub, Port);
}


/*
 *  bWriteAWord
 *
 *  Description:    This routine writes a word to serial EEPROM,
 *                  and then check if the target word is written.
 *
 *  Inputs:
 *
 *      pPub            - pointer to tPUB data structure.
 *      Port            - Read/write EEPROM port.
 *      wIndex          - word index
 *      wData           - word data
 *
 *  Outputs:
 *
 *      N/A
 *
 *  Returns:
 *
 *      eSUCCESS/eFAIL
 *
 */
tRESULT
bWriteAWord(
    tPPUB       pPub,
    tPORT       Port,
    tUSHORT     wIndex,
    tUSHORT     wData
    )
{
    tUSHORT     wGepUse;

    WriteEnable(pPub, Port);
    Erase(pPub, Port, wIndex);
    WriteWordToEEPROM(pPub, Port, wIndex, wData);
    WriteDisable(pPub, Port);

    /* Check written data */
    if (ReadAWord(pPub, Port, wIndex, &wGepUse) == eFAIL)
        return eFAIL;
    if (wData != wGepUse)
        return eFAIL;

    return eSUCCESS;
}

#endif


/*
 *  bReadSROMWords
 *
 *  Description:    This routine can read words from serial EEPROM.
 *
 *  Inputs:
 *
 *      pPub            - pointer to tPUB data structure.
 *      Port            - Read/Write EEPROM port.
 *      pBuffer         - preallocated buffer for storing read data.
 *      nLen            - Words to read.
 *
 *  Outputs:
 *
 *      pBuffer
 *
 *  Returns:
 *
 *      eSUCCESS/eFAIL
 *
 */
tRESULT
bReadSROMWords(
    tPPUB       pPub,
    tPORT       Port,
    tPUSHORT    pBuffer,
    tUINT       nLen
    )
{
    tUSHORT     wIndex;

    MakeSureSROMIsIdle(pPub, Port);

    for (wIndex=0; nLen; nLen--, wIndex++)
    {
        if (bReadAWord(pPub, Port, wIndex, pBuffer)==eFAIL)
            return eFAIL;
        else
            pBuffer++;
    }
    return eSUCCESS;
}
